@@ -5,15 +5,17 @@ import (
 	"testing"
 	"time"
 
-	"github.com/operator-framework/operator-registry/pkg/api"
-	opregistry "github.com/operator-framework/operator-registry/pkg/registry"
 	"github.com/stretchr/testify/require"
 	corev1 "k8s.io/api/core/v1"
 	rbacv1 "k8s.io/api/rbac/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	k8sfake "k8s.io/client-go/kubernetes/fake"
 	"k8s.io/client-go/tools/cache"
 
+	"github.com/operator-framework/operator-registry/pkg/api"
+	opregistry "github.com/operator-framework/operator-registry/pkg/registry"
+
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned/fake"
@@ -43,13 +45,15 @@ func TestNamespaceResolver(t *testing.T) {
 	namespace := "catsrc-namespace"
 	catalog := CatalogKey{"catsrc", namespace}
 	type out struct {
-		steps [][]*v1alpha1.Step
-		subs  []*v1alpha1.Subscription
-		err   error
+		steps   [][]*v1alpha1.Step
+		lookups []v1alpha1.BundleLookup
+		subs    []*v1alpha1.Subscription
+		err     error
 	}
 	nothing := out{
-		steps: [][]*v1alpha1.Step{},
-		subs:  []*v1alpha1.Subscription{},
+		steps:   [][]*v1alpha1.Step{},
+		lookups: []v1alpha1.BundleLookup{},
+		subs:    []*v1alpha1.Subscription{},
 	}
 	tests := []struct {
 		name         string
@@ -98,6 +102,37 @@ func TestNamespaceResolver(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "SingleNewSubscription/ResolveOne/BundlePath",
+			clusterState: []runtime.Object{
+				newSub(namespace, "a", "alpha", catalog),
+			},
+			querier: NewFakeSourceQuerier(map[CatalogKey][]*api.Bundle{
+				catalog: {
+					bundle("a.v1", "a", "alpha", "", nil, Requires1, nil, nil),
+					stripManifests(withBundlePath(bundle("b.v1", "b", "beta", "", Provides1, nil, nil, nil), "quay.io/test/bundle@sha256:abcd")),
+				},
+			}),
+			out: out{
+				steps: [][]*v1alpha1.Step{
+					bundleSteps(bundle("a.v1", "a", "alpha", "", nil, Requires1, nil, nil), namespace, "", catalog),
+					subSteps(namespace, "b.v1", "b", "beta", catalog),
+				},
+				lookups: []v1alpha1.BundleLookup{
+					{
+						Path: "quay.io/test/bundle@sha256:abcd",
+						Identifier: "b.v1",
+						CatalogSourceRef: &corev1.ObjectReference{
+							Namespace: catalog.Namespace,
+							Name:      catalog.Name,
+						},
+					},
+				},
+				subs: []*v1alpha1.Subscription{
+					updatedSub(namespace, "a.v1", "a", "alpha", catalog),
+				},
+			},
+		},
 		{
 			name: "SingleNewSubscription/ResolveOne/AdditionalBundleObjects",
 			clusterState: []runtime.Object{
@@ -188,6 +223,31 @@ func TestNamespaceResolver(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "InstalledSub/UpdateAvailable/FromBundlePath",
+			clusterState: []runtime.Object{
+				existingSub(namespace, "a.v1", "a", "alpha", catalog),
+				existingOperator(namespace, "a.v1", "a", "alpha", "", Provides1, nil, nil, nil),
+			},
+			querier: NewFakeSourceQuerierCustomReplacement(catalog, stripManifests(withBundlePath(bundle("a.v2", "a", "alpha", "a.v1", Provides1, nil, nil, nil), "quay.io/test/bundle@sha256:abcd"))),
+			out: out{
+				steps: [][]*v1alpha1.Step{},
+				lookups: []v1alpha1.BundleLookup{
+					{
+						Path:     "quay.io/test/bundle@sha256:abcd",
+						Identifier: "a.v2",
+						Replaces: "a.v1",
+						CatalogSourceRef: &corev1.ObjectReference{
+							Namespace: catalog.Namespace,
+							Name:      catalog.Name,
+						},
+					},
+				},
+				subs: []*v1alpha1.Subscription{
+					updatedSub(namespace, "a.v2", "a", "alpha", catalog),
+				},
+			},
+		},
 		{
 			name: "InstalledSub/NoRunningOperator",
 			clusterState: []runtime.Object{
@@ -349,55 +409,6 @@ func TestNamespaceResolver(t *testing.T) {
 				},
 			},
 		},
-		{
-			// This test verifies that ownership of an api can be migrated between two operators
-			name: "OwnedAPITransfer",
-			clusterState: []runtime.Object{
-				existingSub(namespace, "a.v1", "a", "alpha", catalog),
-				existingOperator(namespace, "a.v1", "a", "alpha", "", Provides1, nil, nil, nil),
-				existingSub(namespace, "b.v1", "b", "alpha", catalog),
-				existingOperator(namespace, "b.v1", "b", "alpha", "", nil, Requires1, nil, nil),
-			},
-			querier: NewFakeSourceQuerier(map[CatalogKey][]*api.Bundle{
-				catalog: {
-					bundle("a.v2", "a", "alpha", "a.v1", nil, nil, nil, nil),
-					bundle("b.v2", "b", "alpha", "b.v1", Provides1, nil, nil, nil),
-				},
-			}),
-			out: out{
-				steps: [][]*v1alpha1.Step{
-					bundleSteps(bundle("a.v2", "a", "alpha", "a.v1", nil, nil, nil, nil), namespace, "", catalog),
-					bundleSteps(bundle("b.v2", "b", "alpha", "b.v1", Provides1, nil, nil, nil), namespace, "", catalog),
-				},
-				subs: []*v1alpha1.Subscription{
-					updatedSub(namespace, "a.v2", "a", "alpha", catalog),
-					updatedSub(namespace, "b.v2", "b", "alpha", catalog),
-				},
-			},
-		},
-		{
-			name: "PicksOlderProvider",
-			clusterState: []runtime.Object{
-				newSub(namespace, "b", "alpha", catalog),
-			},
-			querier: NewFakeSourceQuerier(map[CatalogKey][]*api.Bundle{
-				catalog: {
-					bundle("a.v1", "a", "alpha", "", Provides1, nil, nil, nil),
-					bundle("a.v2", "a", "alpha", "a.v1", nil, nil, nil, nil),
-					bundle("b.v1", "b", "alpha", "", nil, Requires1, nil, nil),
-				},
-			}),
-			out: out{
-				steps: [][]*v1alpha1.Step{
-					bundleSteps(bundle("a.v1", "a", "alpha", "", Provides1, nil, nil, nil), namespace, "", catalog),
-					bundleSteps(bundle("b.v1", "b", "alpha", "", nil, Requires1, nil, nil), namespace, "", catalog),
-					subSteps(namespace, "a.v1", "a", "alpha", catalog),
-				},
-				subs: []*v1alpha1.Subscription{
-					updatedSub(namespace, "b.v1", "b", "alpha", catalog),
-				},
-			},
-		},
 		{
 			name: "InstalledSub/UpdateInHead/SkipRange",
 			clusterState: []runtime.Object{
@@ -414,43 +425,6 @@ func TestNamespaceResolver(t *testing.T) {
 				},
 			},
 		},
-		{
-			// This test uses logic that implements the FakeSourceQuerier to ensure
-			// that the required API is provided by the new Operator.
-			//
-			// Background:
-			// OLM used to add the new operator to the generation before removing
-			// the old operator from the generation. The logic that removes an operator
-			// from the current generation removes the APIs it provides from the list of
-			// "available" APIs. This caused OLM to search for an operator that provides the API.
-			// If the operator that provides the API uses a skipRange rather than the Spec.Replaces
-			// field, the Replaces field is set to an empty string, causing OLM to fail to upgrade.
-			name: "InstalledSubs/ExistingOperators/OldCSVsReplaced",
-			clusterState: []runtime.Object{
-				existingSub(namespace, "a.v1", "a", "alpha", catalog),
-				existingSub(namespace, "b.v1", "b", "beta", catalog),
-				existingOperator(namespace, "a.v1", "a", "alpha", "", nil, Requires1, nil, nil),
-				existingOperator(namespace, "b.v1", "b", "beta", "", Provides1, nil, nil, nil),
-			},
-			querier: NewFakeSourceQuerier(map[CatalogKey][]*api.Bundle{
-				catalog: {
-					bundle("a.v1", "a", "alpha", "", nil, nil, nil, nil),
-					bundle("a.v2", "a", "alpha", "a.v1", nil, Requires1, nil, nil),
-					bundle("b.v1", "b", "beta", "", Provides1, nil, nil, nil),
-					bundle("b.v2", "b", "beta", "b.v1", Provides1, nil, nil, nil),
-				},
-			}),
-			out: out{
-				steps: [][]*v1alpha1.Step{
-					bundleSteps(bundle("a.v2", "a", "alpha", "a.v1", nil, Requires1, nil, nil), namespace, "", catalog),
-					bundleSteps(bundle("b.v2", "b", "beta", "b.v1", Provides1, nil, nil, nil), namespace, "", catalog),
-				},
-				subs: []*v1alpha1.Subscription{
-					updatedSub(namespace, "a.v2", "a", "alpha", catalog),
-					updatedSub(namespace, "b.v2", "b", "beta", catalog),
-				},
-			},
-		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
@@ -466,12 +440,14 @@ func TestNamespaceResolver(t *testing.T) {
 			lister := operatorlister.NewLister()
 			lister.OperatorsV1alpha1().RegisterSubscriptionLister(namespace, informerFactory.Operators().V1alpha1().Subscriptions().Lister())
 			lister.OperatorsV1alpha1().RegisterClusterServiceVersionLister(namespace, informerFactory.Operators().V1alpha1().ClusterServiceVersions().Lister())
+			kClientFake := k8sfake.NewSimpleClientset()
 
-			resolver := NewOperatorsV1alpha1Resolver(lister, clientFake)
-			steps, subs, err := resolver.ResolveSteps(namespace, tt.querier)
+			resolver := NewOperatorsV1alpha1Resolver(lister, clientFake, kClientFake)
+			steps, lookups, subs, err := resolver.ResolveSteps(namespace, tt.querier)
 			require.Equal(t, tt.out.err, err)
 			t.Logf("%#v", steps)
 			RequireStepsEqual(t, expectedSteps, steps)
+			require.ElementsMatch(t, tt.out.lookups, lookups)
 			require.ElementsMatch(t, tt.out.subs, subs)
 		})
 	}
@@ -535,14 +511,15 @@ func TestNamespaceResolverRBAC(t *testing.T) {
 			for _, steps := range tt.out.steps {
 				expectedSteps = append(expectedSteps, steps...)
 			}
+			kClientFake := k8sfake.NewSimpleClientset()
 			clientFake, informerFactory, _ := StartResolverInformers(namespace, stopc, tt.clusterState...)
 			lister := operatorlister.NewLister()
 			lister.OperatorsV1alpha1().RegisterSubscriptionLister(namespace, informerFactory.Operators().V1alpha1().Subscriptions().Lister())
 			lister.OperatorsV1alpha1().RegisterClusterServiceVersionLister(namespace, informerFactory.Operators().V1alpha1().ClusterServiceVersions().Lister())
 
-			resolver := NewOperatorsV1alpha1Resolver(lister, clientFake)
+			resolver := NewOperatorsV1alpha1Resolver(lister, clientFake, kClientFake)
 			querier := NewFakeSourceQuerier(map[CatalogKey][]*api.Bundle{catalog: tt.bundlesInCatalog})
-			steps, subs, err := resolver.ResolveSteps(namespace, querier)
+			steps, _, subs, err := resolver.ResolveSteps(namespace, querier)
 			require.Equal(t, tt.out.err, err)
 			RequireStepsEqual(t, expectedSteps, steps)
 			require.ElementsMatch(t, tt.out.subs, subs)
